#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "../fstrfind/fstrfind.h"

/**************************************
  See fstrfind.h for calling instructions.

  References:
  http://www.geekpedia.com/tutorial38_Searching-for-a-string-in-a-File.html
  This is similar to what I need, but not quite it, so I wrote it myself.

  Someone there mentioned the Knuth-Morris-Pratt algorithm,
  http://en.wikipedia.org/wiki/Knuth-Morris-Pratt_algorithm
  which could certainly be useful. But it might be hard to
  implement in combination with the ability to skip multiple
  white space chars, and ignore whitespace after < and before >
  in HTML files. So I used a simpler and slower, but more
  straightforward approch instead.

 *************************************/
long fstrfindex (FILE *fp, char *s,
   int NotCaseSensitive,
   int IgnoreWhiteSpace,
   int HtmlTagSpace)
{
   int c;
   long pos, p;
   char *t;

   for (
      t = s, p = pos = ftell(fp), c = getc(fp);
      *t && c != EOF;
      p = ftell(fp), c = getc(fp))
   {
      #ifdef MAIN
         fprintf(stdout, 
            "Comparing    %c (%02x) at %5d in buffer\n",
            *t, *t, t - s);
         fprintf(stdout, 
            "\twith %c (%02x) at %5ld in file\n", c, c, (long)p);
      #endif

      if (*t != c && !
         (NotCaseSensitive && tolower(*t) == tolower(c))
         )
      {
         t = s;
         pos = ++p;
         fseek(fp, pos, SEEK_SET);
      }
      else
      {
         t++;
      }
   }
   return *t ? -1 : pos;
}


/**************************************
   Test driver
 *************************************/
#ifdef MAIN
// char *testfilename = "fstrtest.htm";
char *testfilename = "destaqnl.htm";
//char *searchstring = "<A HREF";
char *searchstring = "<hr>";

int main (int argc, char **argv)
{
   FILE *fp;
   long pos;
   char buf[72 + 1];

   if (argc > 1 && argv[1] && *argv[1])
       searchstring = argv[1];   

#if defined(_Windows) || defined (__MSDOS__)
   if ((fp = fopen(testfilename, "rb")) == NULL)
#else
   if ((fp = fopen(testfilename, "r")) == NULL)
#endif
   {
      fprintf(stdout, "Can't open %s\n", testfilename);
      return 2;
   }

   fseek(fp, 624, SEEK_SET);
// fseek(fp, 1624, SEEK_SET);
   pos = fstrfindex(fp, searchstring, 1, 0, 0);
   
   if (pos < 0)
   {
      printf("String \"%s\" not found\n", searchstring);
   }
   else
   {
      printf("String \"%s\" found at file position %ld\n",
         searchstring, (long)pos);
      fseek(fp, pos, SEEK_SET);
      memset(buf, '\0', sizeof buf);
      fread(buf, sizeof buf - 1, 1, fp);
      printf("File contains:\n%-72.72s\n", buf);
   }
   
   fclose(fp);

   return 0;
}

#endif

